///////////////////////////////////////////////////////////////
//                                                           //
//  data.cpp   -   Implementation file for the data class    //
//  Language:      Visual C++, ver 5.0                       //
//  Platform:      Micron Dual Pentium 200, Windows NT 4.0   //
//  Author:        M. A. Saleh Elmohamed                     //
//                                                           //
///////////////////////////////////////////////////////////////
#include "data.h"

data::data()
{
}

//--- Size Constructor -------------------
data::data(char* filename, int numInputs, int numOutputs)
{
  read(filename, numInputs, numOutputs);
}


//------ Destructor ---------
data::~data()
{
}

//-------- Read data from a file ---------
int data::read(char* filename, int numInputs, int numOutputs)
{
  int i = 0;
  vector<double> data_vec(numInputs + numOutputs);

  mi_numInputs = numInputs;
  mi_numOutputs = numOutputs;

  ifstream infile(filename);
  while(infile.good())
    {
      //reads one line from the data file
      for(i=0; (i<(numInputs + numOutputs))&&infile.good(); i++)
	infile >> data_vec[i];

      if(!infile.good())
	return size();
		
      //Puts the vector into the data set
      mvvd_dataSet.push_back(data_vec);
    }
  return size();
}

//----- Access an input set from the input - output pair ----
vector<double>& data::input(int index)
{
  static vector<double> safe(mi_numInputs);
  if(index<0 || index>mvvd_dataSet.size())
    {
      cerr << "Out of bounds";
      return safe;
    }
  for(int i=0; i<mi_numInputs; i++)
    {
      safe[i] = mvvd_dataSet[index][i];
    }
  return safe;
}

//---- Access an output set from the input - output pair ----
vector<double>& data::output(int index)
{
  static vector<double> safe(mi_numOutputs);
  if(index<0 || index>mvvd_dataSet.size())
    {
      cerr << "Out of bounds";
      return safe;
    }
  for(int i=mi_numInputs, j=0; i<(mi_numInputs + mi_numOutputs); i++, j++)
    {
      safe[j] = mvvd_dataSet[index][i];
    }
  return safe;
}

//---- Returns the size of the data set ----
int data::size()
{
  return(mvvd_dataSet.size());
}

//---- Randomly shuffle the data set ----
void data::scramble()
{
  random_shuffle(mvvd_dataSet.begin(), mvvd_dataSet.end()) ;
}

//--------- Test Stub ---------------
#ifdef TEST_DATA
void main()
{
  int i;
  data train_data("p1train.dat");

  cout << "Before scrambling :" << endl;
  for(i=0; i<train_data.size(); i++)
    cout << train_data.input(i)[0] << "\t" << train_data.output(i)[0] << endl;

  train_data.scramble();

  cout << endl << "After scrambling :" << endl;
  for(i=0; i<train_data.size(); i++)
    cout << train_data.input(i)[0] << "\t" << train_data.output(i)[0] << endl;

}
#endif
