///////////////////////////////////////////////////////////////
//                                                           //
//  data.h  -    Manages the data to be fed to the neural net//
//  Language:      Visual C++, ver 5.0                       //
//  Platform:      Micron Dual Pentium 200, Windows NT 4.0   //
//  Author:        M. A. Saleh Elmohamed                     //
//                                                           //
///////////////////////////////////////////////////////////////
/*
  Manual Page
  -----------

  Data Operations:
  ----------------
  
  1) data test_data("test_file1.dat", 1, 1); 
    - Constructs a data set from the file test_file1.dat
    with one input value and one output value in each row.

  2) test_data.read("test_file2.dat", 3, 2)
    - Reads test_file2.dat and creates the data set to have
    3 inputs and 2 outputs in each row.

  3) vector<double> inputs = test_data.input(5);
    - Returns the input values in the 5th row as a vector.

  4) vector<double> outputs = test_data.output(5);
    - Returns the output values in the 5th row as a vector.

  5) int s = test_data.size();
    - Returns the size of the data set.

  6) test_data.scramble();
    - Randomly shuffles the rows in the data set.
*/
///////////////////////////////////////////////////////////////
//                                                           //
//                      maintenance page                     //
//                                                           //
///////////////////////////////////////////////////////////////
//  Build Process                                            //
//                                                           //
//  Files Required:                                          //
//    data.h, data.cpp                                       //
//                                                           //
//  Building with Visual C++ , ver 5.0, from command line:   //
//    cl -GX -DTEST_DATA data.cpp                            //
//                                                           //
///////////////////////////////////////////////////////////////
/*
  Maintenance History
  ===================
  ver 1.0 : 26 June 98
  - first release
*/
///////////////////////////////////////////////////////////////
//                                                           //
//                         Declarations                      //
//                                                           //
///////////////////////////////////////////////////////////////
#ifndef DATA_H
#define DATA_H

#include <iostream.h>
#include <fstream.h>
#include <vector>
#include <algorithm>
#include <functional>

using namespace std;

class data
{
 private:
  vector< vector<double> > mvvd_dataSet;
  int mi_numInputs;
  int mi_numOutputs;

 public:
  data();
  data(char* filename, int numInputs=1, int numOutputs=1);
  ~data();

  int read(char* filename, int numInputs=1, int numOutputs=1);

  vector<double>& input(int index);
  vector<double>& output(int index);

  int size();
  void scramble();
};

#endif //for #ifndef DATA_H
