///////////////////////////////////////////////////////////////
//                                                           //
//  layer.h   -    Models a layer of neurons                 //
//  Language:      Visual C++, ver 5.0                       //
//  Platform:      Micron Dual Pentium 200, Windows NT 4.0   //
//  Author:        M. A. Saleh Elmohamed                     //
///////////////////////////////////////////////////////////////
/*
  Manual Page
  -----------

  Layer Operations:
  -----------------
  The layer class models a layer of neurons

  Layer Operations:
  1) layer test_layer(3,3); 
     - Constructs a 3 input layer with 3 neurons.

  2) vector<double> ins;
     test_layer.set_inputs(ins);
     - Sets the inputs of the layer

  3) vector< vector<double> > weights;
     test_layer.set_weights(weights);
     - Sets the weights of the neurons in the layer

  4) vector< vector<double> > weights;
     weights = test_layer.get_weights();
     - Gets the weights of the neurons in the layer

  5) vector<double> bias;
     test_layer.set_bias(bias);
     - Sets the bias of the neurons in the layer

  6) vector<double> bias;
     bias = test_layer.get_bias();
     - Gets the bias of the neurons in the layer

  7) vector<double> outputs = test_layer.get_output();
     - Returns the layer outputs.


  8) vector<double> p_vec = test_layer.get_propTermVec();
     - Returns the propogation term vector for the layer.
     (The Sigma term in Eqn. 9.7 for a non-output node)

  9) vector<double> errTerm_vec = test_layer.get_error_terms(propTerm_vec);
    - Returns the error terms(Epsilon) vector for the layer.

  10) test_layer.adapt();
    - Adapts the layer by updating the neuron weights.
*/
///////////////////////////////////////////////////////////////
//                                                           //
//                      maintenance page                     //
//                                                           //
///////////////////////////////////////////////////////////////
//  Build Process                                            //
//                                                           //
//  Files Required:                                          //
//    layer.h, layer.cpp                                     //
//                                                           //
//  Building with Visual C++ , ver 5.0, from command line:   //
//    cl -GX -DTEST_LAYER layer.cpp                          //
//                                                           //
///////////////////////////////////////////////////////////////
/*
  Maintenance History
  ===================
  ver 1.0 : 26 June 98
  - first release
*/
///////////////////////////////////////////////////////////////
//                                                           //
//                         Declarations                      //
//                                                           //
///////////////////////////////////////////////////////////////
#ifndef LAYER_H
#define LAYER_H

#include "neuron.h"

class layer
{
 private:
  int mi_numInputs;
  int mi_numNeurons;
  vector<double> mvd_output;
  vector<neuron> mvn_neurons;
  bool mb_outputLayer; //Is this an output layer ?

 public:
  layer();
  layer(int numInputs, int numNeurons, bool outputLayer=false);
  ~layer();
  void set_inputs(vector<double> input_vec);
  void set_weights(vector< vector<double> > weight_vec);
  vector< vector<double> > get_weights();

  void set_bias(vector<double> bias_vec);
  vector<double> get_bias();

  vector<double> get_outputs();
  vector<double> get_propTermVec();
  vector<double> get_error_terms(vector<double> propTerm_vec);
  void adapt();

  //Required by STL vector class
  bool operator==(const layer& l) const;
  bool operator<(const layer& l) const;
  layer& operator=(const layer& l);
};

#endif
