///////////////////////////////////////////////////////////////
//                                                           //
//  network.h -    Models a neural network using one or more //
//                                 layers                    //
//  Language:      Visual C++, ver 5.0                       //
//  Platform:      Micron Dual Pentium 200, Windows NT 4.0   //
//  Author:        M. A. Saleh Elmohamed                     //
///////////////////////////////////////////////////////////////
/*
  Manual Page
  -----------

  Network Operations:
  -------------------
  The network class models a neural network

  1) network test_network(3, 1, 2, 3); 
     - Constructs a 3 input, 1 output network with 2 
     hidden layers and 3 neurons in each hidden layer.

  2) vector< layer_weights > weight_vec;
     test_network.set_weights(weight_vec);
     - Sets the weights of the neurons in the network.

  3) weight_vec = test_network.get_weights();
     - Gets the weights of the neurons in the network.

  4) vector< vector<double> > bias_vec;
     test_network.set_bias(bias_vec);
     - Sets the bias of the neurons in the network.

  5) bias_vec = test_network.get_bias();
     - Gets the bias of the neurons in the network.

  6) test_network.train(input_vec, desired_output_vec);
     - Trains the network for an input - output pair

  7) output_vec = test_network.get_outputs(input_vec)
     - Gets the output of the network for the given input.
*/
///////////////////////////////////////////////////////////////
//                                                           //
//                      maintenance page                     //
//                                                           //
///////////////////////////////////////////////////////////////
//  Build Process                                            //
//                                                           //
//  Files Required:                                          //
//    network.h, network.cpp                                 //
//                                                           //
//  Building with Visual C++ , ver 5.0, from command line:   //
//    cl -GX -DTEST_NETWORK network.cpp                      //
//                                                           //
///////////////////////////////////////////////////////////////
/*
  Maintenance History
  ===================
  ver 1.0 : 27 June 98
  - first release
*/
///////////////////////////////////////////////////////////////
//                                                           //
//                         Declarations                      //
//                                                           //
///////////////////////////////////////////////////////////////
#ifndef NETWORK_H
#define NETWORK_H

#pragma warning( disable : 4786 )
#include "layer.h"

typedef vector< vector<double> > layer_weights;

class network
{
 private:
  int mi_numInputs;
  int mi_numOutputs;
  int mi_numHiddenLayers;
  int mi_numHiddenLayerNodes;

  layer ml_outputLayer;
  vector<layer> mvl_hiddenLayer;
  bool mb_training;

 public:
  network(int numInputs, 
	  int numOutputs, 
	  int numHiddenLayers = 1, 
	  int numHiddenLayerNodes = 1);

  ~network();

  void set_weights(vector< layer_weights >  weight_vec);
  vector< layer_weights > get_weights();
  
  void set_bias(vector< vector<double> > bias_vecs);
  vector< vector<double> > get_bias();

  vector<double> train(vector<double> input_vec, 
		       vector<double> desired_output_vec);

  vector<double> get_outputs(vector<double> input_vec);

};

#endif
