///////////////////////////////////////////////////////////////
//                                                           //
//  neuron.h  -    Models a neuron                           //
//  Language:      Visual C++, ver 5.0                       //
//  Platform:      Micron Dual Pentium 200, Windows NT 4.0   //
//  Author:        M. A. Saleh Elmohamed                     //
///////////////////////////////////////////////////////////////
/*
  Manual Page
  -----------

  Neuron Operations:
  ------------------
  The neuron class models a basic neuron

  Neuron Operations:
  1) neuron test_neuron(3); 
     - Constructs a 3-input neuron.

  2) test_neuron.input(1) = 0.9871;
     double d = test_neuron.input(1);
     - Access a neuron input.

  3) test_neuron.weight(1) = 0.9871;
     double d = test_neuron.weight(1);
     - Access a neuron weight.

  4) test_neuron.bias() = 0.9871;
     double d = test_neuron.bias();
     - Access the neuron's bias.

  5) double d = test_neuron.find_output();
     - Calculates the neuron o/p.

  6) double d = test_neuron.output();
     - Returns the last calculated neuron o/p, without 
     recalculating.

  7) double d = test_neuron.find_error_term(x);
     - Calculates the neuron's error_term - Epsilon
     (For info about the argument 'x', see the function 
      definition prologue) 

  8) double d = test_neuron.error_term();
     - Returns the last calculated error term, without recalculating. 

  9) test_neuron.update_weights();
     - Updates the weights of the neuron.
*/
///////////////////////////////////////////////////////////////
//                                                           //
//                      maintenance page                     //
//                                                           //
///////////////////////////////////////////////////////////////
//  Build Process                                            //
//                                                           //
//  Files Required:                                          //
//    neuron.h, neuron.cpp                                   //
//                                                           //
//  Building with Visual C++ , ver 5.0, from command line:   //
//    cl -GX -DTEST_NEURON neuron.cpp                        //
//                                                           //
///////////////////////////////////////////////////////////////
/*
  Maintenance History
  ===================
  ver 1.0 : 26 June 98
  - first release
*/
///////////////////////////////////////////////////////////////
//                                                           //
//                         Declarations                      //
//                                                           //
///////////////////////////////////////////////////////////////
#ifndef NEURON_H
#define NEURON_H

#include<iostream.h>
#include<math.h>
#include<vector>

using namespace std;

const double INITIAL_WEIGHT = 0.1;
const double INITIAL_BIAS = 0.0001;
const double ETA = 0.01;

class neuron
{
 private:
  int mi_numInputs;
  vector<double> mvd_input;  // Vector of neuron inputs
  vector<double> mvd_weight; // Vector of input weights
  double md_bias;
  double md_output;
  double md_errorTerm; // Epsilon
  bool mb_outputNode;  // Is the neuron an output node ?

 public:
  neuron();
  neuron(int n,
	 bool out_node=false,
	 double initial_weight=INITIAL_WEIGHT, 
	 double initial_bias=INITIAL_BIAS);
  ~neuron();

  double& input(int index);
  double& weight(int index);
  double& bias();

  double find_output();
  double output();

  double find_error_term(double term);
  double error_term();

  void update_weights();

  //---Required by the STL vector class ---//
  bool operator==(const neuron& n) const;
  bool operator<(const neuron& n) const;
  neuron& operator=(const neuron& n);
};

#endif
