///////////////////////////////////////////////////////////////
//                                                           //
//  nnet.cpp   -   Implementation file for the neural        //
//                 network application                       //
//  Language:      Visual C++, ver 5.0                       //
//  Platform:	   Micron Dual Pentium 200, Windows NT 4.0   //
//  Author:        M. A. Saleh Elmohamed                     //
///////////////////////////////////////////////////////////////

#include <strstream.h>
#include <mpi.h>
#include <math.h>
#include <stdlib.h>
#include <string> 
#include "data.h"
#include "network.h"
#include <time.h>

using namespace std;

#define INDEX myid + numprocs*i

const int numInputs = 14;
const int numOutputs = 1;
const int numHiddenLayers = 2;
const int numHiddenLayerNodes = 7;
const int numIterations = 1000;
const bool training = false;
static 	int numprocs, myid;

// Function prototypes - Forward references
static void train(char *filname, char *bias, char *weights, char *error);
static void test(char *filname, char *bias, char *weights, char *error);
static void WriteWeightsToFile(vector< layer_weights >, char* filename);
static void ReadWeightsFromFile(vector< layer_weights >&, char* filename);
static void WriteBiasToFile(vector< vector<double> >, char* filename);
static void ReadBiasFromFile(vector< vector<double> >&, char* filename);


//------ The neural net application--------
int main(int argc, char *argv[])
{
  MPI_Init(&argc, &argv);   //MPI command...
  MPI_Comm_size(MPI_COMM_WORLD, &numprocs);  //MPI command...
  MPI_Comm_rank(MPI_COMM_WORLD, &myid);      //MPI command...

  cout << "Process : " << myid << endl;
  data train_data("train1.dat", 14, 1);
	
  time_t tStart, tTrain, tTest;
  long TrainTime, TestTime;

  string filname = "train";
  filname += (char)(48 + myid);

  string resfname = "result";
  resfname += (char)(48 + myid);

  ofstream trainfile(filname.c_str());
	
  for (int i=myid; i < train_data.size(); i+=numprocs)
    {
      for ( int j = 0; j < train_data.input(i).size(); j++)
	{
	  trainfile << train_data.input(i)[j] << " ";
	}
      trainfile << train_data.output(i)[0] << endl;
    }

  string biasfile = "bias";
  biasfile += (char)(48 + myid);

  string weightsfile = "weights";
  weightsfile += (char)(48 + myid);

  string errorfile = "error";
  errorfile += (char)(48 + myid);

  //if(training){

  time (&tStart);
  train(filname.c_str(), biasfile.c_str(), weightsfile.c_str(), errorfile.c_str());
  time (&tTrain);
  TrainTime = tTrain - tStart;
  
  //}
  //else{

  time (&tStart);
  test(resfname.c_str(),biasfile.c_str(), weightsfile.c_str(), errorfile.c_str());
  time (&tTest);
  TestTime = tTest - tStart;

  //}

  if (myid == 0){
    cout << endl << "------------------------------------------------" << endl;
    cout << "Time taken to train networks : " << TrainTime << endl;
    cout << "Time taken to test networks : " << TestTime << endl;
    cout << "--------------------------------------------------------" << endl;
  }
  
  MPI_Finalize();   // MPI command ...
  return 0;
}

//------- Train the network ---------
void train(char *filname, char *bias, char *weights, char *error)
{
  int i,j;
  double err_sum = 0;
  double test_op;

  vector<double> output(numOutputs); //To receive the o/p vector
  vector< layer_weights > nw_weights(numHiddenLayers+1); //To hold the n/w weights
  vector< vector<double> > nw_bias(numHiddenLayers+1); //To hold the n/w bias

  network nnet(numInputs, numOutputs, numHiddenLayers, numHiddenLayerNodes);

  ofstream error_file(error); // To write the squared error measure
  nw_weights = nnet.get_weights();
  ReadWeightsFromFile(nw_weights, "ini_weights.dat");
  nnet.set_weights(nw_weights);
  data training_set(filname, 14, 1); //Build data set from 'p2train.dat'
  for(i=0; i < numIterations; i++)
    {
      training_set.scramble(); // Shuffle the data set before each iteration
      err_sum = 0;
      for(j=0; j<training_set.size(); j++)
	{
	  output=nnet.train(training_set.input(j), training_set.output(j));
	  test_op = output[0];
	  err_sum += pow((training_set.output(j)[0] - output[0]),2);
	}
      //averaging the squared error measure
      cout << err_sum/training_set.size() << endl;       

      //for the data set after each iteration and writing it to the file
      error_file << err_sum/training_set.size() << endl; 
    }

  //This helps to give the size of the vector to read from the file  
  nw_weights = nnet.get_weights();

  WriteWeightsToFile(nw_weights, weights);
  nw_bias = nnet.get_bias();
  WriteBiasToFile(nw_bias, bias);
}

//------------ Test the network ----------------
void test(char *filname, char *bias, char *weights, char *error)
{
  vector<double> output(numOutputs); //To receive the o/p vector
  vector< layer_weights > nw_weights(numHiddenLayers+1); //To hold the n/w weights
  vector< vector<double> > nw_bias(numHiddenLayers+1); //To hold the n/w bias

  //Construct a neural net with the required parameters
  network nnet(numInputs, numOutputs, numHiddenLayers, numHiddenLayerNodes);


  //Into this I'm writing the i/p, desired o/p & actual o/p
  ofstream result_file("test_result.dat"); 

  cout << "Loading test data....Please wait...." << endl;
  data testing_set("test.dat", 14, 1);

  //Helps in fixing the size of the weights vector
  nw_weights = nnet.get_weights(); 
  ReadWeightsFromFile(nw_weights, weights);
  nnet.set_weights(nw_weights);

  nw_bias = nnet.get_bias();
  ReadBiasFromFile(nw_bias, bias);
  nnet.set_bias(nw_bias);

  double rootop, localop;

  ostrstream sout;
  sout << "   Day   Hour  Temp. Hum.  Wind Cloud  Temp. Hum.  Wind Cloud  Temp. Hum.  Wind Cloud       Load" << endl;
  sout << "                                                                                     (Desired) (Actual)" << endl;
  sout << "--------------------------------------------------------" << endl;
  sout << ends;
  cout << sout.str();
  result_file << sout.str();

  //Test the n/w and log the results
  for(int i=0; i<testing_set.size(); i++)
    {
      ostrstream sout1;
      output=nnet.get_outputs(testing_set.input(i)); //Apply i/p & get o/p

      localop = output[0];

      // MPI command ...
      MPI_Reduce (&localop, &rootop, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
      if( myid == 0)
	{
	  rootop /= numprocs;
	  
	  for(int j=0; j<14; j++)
	    {
	      sout1.width(6); sout1 << testing_set.input(i)[j];
	    }
	  sout1.width(8); sout1 << testing_set.output(i)[0] ;
	  sout1.flags(ios::right|ios::showpoint|ios::fixed);
	  sout1.width(11); sout1 << rootop << endl << ends;
	  cout << sout1.str();
	}
      result_file << sout1.str();
    }
  
  ostrstream sout2;
  sout2 << endl << "This o/p served only as the raw data for importing into MS Excel." << endl;
  sout2 << "Once imported into Excel, the o/p was appropriately scaled and analyzed." << endl << ends;
  cout << sout2.str();
  result_file << sout2.str();
}

//------- Writes weights to a file -------------
void WriteWeightsToFile(vector< layer_weights > nw_weights, char* filename)
{
  ofstream out_file(filename);
  for(int i=0; i<(numHiddenLayers+1); i++)
    for(int j=0; j<nw_weights[i].size(); j++)
      for(int k=0; k < nw_weights[i][j].size(); k++)
	out_file << nw_weights[i][j][k] << endl;
}

//------- Reads weights from a file -------------
void ReadWeightsFromFile(vector< layer_weights >& nw_weights, char* filename)
{
  ifstream in_file(filename);
  for(int i=0; i<(numHiddenLayers+1); i++)
    for(int j=0; j<nw_weights[i].size(); j++)
      for(int k=0; k < nw_weights[i][j].size(); k++)
	in_file >> nw_weights[i][j][k];
}

//------- Writes bias to a file -------------
void WriteBiasToFile(vector< vector<double> > nw_bias, char* filename)
{
  ofstream out_file(filename);
  for(int i=0; i<(numHiddenLayers+1); i++)
    for(int j=0; j<nw_bias[i].size(); j++)
      out_file << nw_bias[i][j] << endl;
}

//------- Reads bias from a file -------------
void ReadBiasFromFile(vector< vector<double> >& nw_bias, char* filename)
{
  ifstream in_file(filename);
  for(int i=0; i<(numHiddenLayers+1); i++)
    for(int j=0; j<nw_bias[i].size(); j++)
      in_file >> nw_bias[i][j];
}
